#!/usr/bin/perl 
#
# Sensor for gathering Apache performance data for Zabbix via
# apache mod_status.
# Copyright (c) 2008 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Changes and Modifications
# * Thu May 15 03:40:39 CEST 2008       
#       - script created
#

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd";

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

# fetch status from mod_status
alarm 3;
my $cmd = '/usr/bin/curl -A "mozilla/4.0 (compatible; cURL 7.10.5-pre2; Linux 2.4.20)" -m 12 -s -L -k -b /tmp/bbapache_cookiejar.curl -c /tmp/bbapache_cookiejar.curl -H "Pragma: no-cache" -H "Cache-control: no-cache" -H "Connection: close" "http://localhost/server-status?auto"';
my $server_status = qx($cmd);
if (!( $server_status =~ /Uptime/ )) { print"0\n"; exit(-1); }

# get information from status output
alarm 3;
my($total_accesses,$total_kbytes,$cpuload,$uptime, $reqpersec,$bytespersec,$bytesperreq,$busyservers, $idleservers, $scoreboard);
$total_accesses = $1 if ($server_status =~ /Total\ Accesses:\ ([\d|\.]+)/ig)||0;
$total_kbytes = $1 if ($server_status =~ /Total\ kBytes:\ ([\d|\.]+)/gi);
$cpuload = $1 if ($server_status =~ /CPULoad:\ ([\d|\.]+)/gi);
$uptime = $1 if ($server_status =~ /Uptime:\ ([\d|\.]+)/gi);
$reqpersec = $1 if ($server_status =~ /ReqPerSec:\ ([\d|\.]+)/gi);
$bytespersec = $1 if ($server_status =~ /BytesPerSec:\ ([\d|\.]+)/gi);
$bytesperreq = $1 if ($server_status =~ /BytesPerReq:\ ([\d|\.]+)/gi);
$busyservers = $1 if ($server_status =~ /BusyWorkers:\ ([\d|\.]+)/gi);
$idleservers = $1 if ($server_status =~ /IdleWorkers:\ ([\d|\.]+)/gi);
$scoreboard = $1 if ($server_status =~ /Scoreboard:\ ([A-Z_]+)/gi);

# send information to Zabbix server
alarm 5;
if ( qx($zabbix_send_command -k apache.total_accesses -o $total_accesses) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.total_kbytes -o $total_kbytes) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.cpuload -o $cpuload) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.uptime -o $uptime) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.reqpersec -o $reqpersec) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.bytespersec -o $bytespersec) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.bytesperreq -o $bytesperreq) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.busyservers -o $busyservers) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k apache.idleservers -o $idleservers) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
#if ( qx($zabbix_send_command -k apache.scoreboard -o $scoreboard) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
print "1\n";
exit(0);
# EOF

